import React, { forwardRef, useState } from 'react';
import { Input, Box, Stack, Text, Flex } from '@chakra-ui/react';
import { useTranslation } from 'react-i18next';

import { CAR_COLORS } from './helper';

interface CarColorSelectProps {
  value?: string;
  onChange?: (event: React.ChangeEvent<HTMLInputElement>) => void;
  name?: string;
  isInvalid?: boolean;
}

export const CarColorSelect = forwardRef<HTMLInputElement, CarColorSelectProps>(
  function CarColorSelect(props, ref) {
    const [customColor, setCustomColor] = useState('');
    const [isCustom, setIsCustom] = useState(false);

    const handleColorChange = (value: string) => {
      if (value === 'custom') {
        setIsCustom(true);
        return;
      }
      setIsCustom(false);
      props.onChange?.({
        target: { value },
      } as React.ChangeEvent<HTMLInputElement>);
    };

    const handleCustomColorChange = (
      e: React.ChangeEvent<HTMLInputElement>,
    ) => {
      const value = e.target.value;
      setCustomColor(value);
      props.onChange?.({
        target: { value },
      } as React.ChangeEvent<HTMLInputElement>);
    };

    const { t } = useTranslation('~', {
      keyPrefix: 'dry-wash.order-create.car-color-select',
    });

    const currentValue = isCustom ? 'custom' : props.value;

    return (
      <Stack spacing={4} width='100%' ref={ref}>
        <Flex gap={2} wrap='wrap' pb={2}>
          {CAR_COLORS.map(({ name, code }) => (
            <Box
              key={name}
              flexShrink={0}
              as='button'
              type='button'
              onClick={() => handleColorChange(name)}
            >
              <Flex
                align='center'
                gap={2}
                p={1}
                borderRadius='full'
                borderWidth='2px'
                borderColor='gray.200'
                bg='white'
                _hover={{
                  borderColor: 'primary.500',
                  bg: 'gray.50',
                }}
                justify='center'
                transition='all 0.2s'
                {...(currentValue === name && {
                  borderColor: 'primary.500',
                  bg: 'primary.50',
                  paddingInlineEnd: 3,
                  _hover: {
                    bg: 'primary.50',
                  },
                })}
              >
                <Flex align='center' gap={2}>
                  <Box
                    w='32px'
                    h='32px'
                    borderRadius='full'
                    bg={code}
                    border='1px'
                    borderColor='gray.200'
                    transition='all 0.2s'
                    boxShadow='none'
                    {...(currentValue === name && {
                      borderColor: 'primary.500',
                      boxShadow: 'sm',
                    })}
                  />
                  {currentValue === name && (
                    <Text fontSize='xs' color='primary.700' fontWeight='medium'>
                      {t(`colors.${name}`)}
                    </Text>
                  )}
                </Flex>
              </Flex>
            </Box>
          ))}
          <Box
            flexShrink={0}
            as='button'
            type='button'
            onClick={() => handleColorChange('custom')}
          >
            <Flex
              align='center'
              gap={2}
              p={1}
              paddingInlineEnd={3}
              borderRadius='full'
              borderWidth='2px'
              borderColor='gray.200'
              bg='white'
              _hover={{
                borderColor: 'primary.500',
                bg: 'gray.50',
              }}
              justify='center'
              transition='all 0.2s'
              {...(isCustom && {
                borderColor: 'primary.500',
                paddingInlineStart: 3,
                bg: 'primary.50',
                _hover: {
                  bg: 'primary.50',
                },
              })}
            >
              {isCustom ? (
                <Flex gap={2} align='center'>
                  <Text fontSize='xs' color='primary.700' fontWeight='medium'>
                    {t('custom-label')}
                  </Text>
                  <Input
                    size='sm'
                    width='120px'
                    value={customColor}
                    onChange={handleCustomColorChange}
                    placeholder={t('placeholder')}
                    onClick={(e) => e.stopPropagation()}
                    borderColor='primary.200'
                    _focus={{
                      borderColor: 'primary.500',
                      boxShadow: '0 0 0 1px var(--chakra-colors-primary-500)',
                    }}
                  />
                </Flex>
              ) : (
                <Flex align='center' gap={2}>
                  <Box
                    w='32px'
                    h='32px'
                    borderRadius='full'
                    bg='gray.100'
                    border='1px'
                    borderColor='gray.200'
                    transition='all 0.2s'
                  />
                  <Text fontSize='xs' color='gray.500'>
                    {t('custom')}
                  </Text>
                </Flex>
              )}
            </Flex>
          </Box>
        </Flex>
      </Stack>
    );
  },
);
